"""Automatized Testing Framework"""

from execute import execute


def main():
    """Entry point for conda execution"""
    # Call the main execution function
    execute()


# Check if the script is being run directly
if __name__ == "__main__":
    # Call the main execution function
    main()
